var init = (function (Gio2_0, GLib2_0, Soup2_4, St1_0) {
  'use strict';

  var require = () =>
    imports.byteArray.toString(imports.byteArray.fromString("_")) !== "_" &&
    (imports.byteArray.toString = String);

  var ByteString = {
  	require: require
  };

  ByteString.require();
  const { fromString, toString } = imports.byteArray;
  const { DataInputStream } = imports.gi.Gio;
  const GLib = imports.gi.GLib;
  const { Message, Session, URI } = imports.gi.Soup;

  class Context {
    /** @param {string} url @param {Partial<Context>?} [ctx] */
    static async fetch(url, ctx) {
      ctx = ctx || {};
      const headers = ctx.headers || {};
      const method = ctx.method || "GET";

      const msg = new Message({ method, uri: new URI(url) });
      for (const name of Object.keys(headers)) {
        const header = headers[name];
        if (header) {
          msg.request_headers.append(name, header);
        }
      }
      if (ctx.body !== undefined) {
        msg.request_body.append(fromString(JSON.stringify(ctx.body)));
      }

      const session = new Session();
      const response = await new Promise(resolve =>
        session.send_async(msg, null, (_, $) => resolve(session.send_finish($)))
      );
      const body = await new Promise(resolve => {
        const stream = new DataInputStream({ base_stream: response });
        stream.read_line_async(GLib.PRIORITY_DEFAULT, null, (_, $) => {
          const [maybe] = stream.read_line_finish($);
          resolve(maybe && maybe.length ? toString(maybe) : "null");
        });
      });

      const responseCtx = new Context();
      try {
        responseCtx.body = JSON.parse(body);
      } catch (_) {
        responseCtx.body = body;
      }
      responseCtx.status = msg.status_code;
      return responseCtx;
    }

    constructor(/** @type {any} */ props) {
      /** @type {any} */ this.body = {};
      /** @type {{ [key: string]: string | undefined }} */ this.headers = {};
      this.id = "";
      this.ip = "";
      this.length = 0;
      this.method = "GET";
      this.path = "";
      this.protocol = "HTTP/1.1";
      this.query = "";
      this.props = props;
      this.status = 200;
      this.userId = "";
    }

    toString() {
      const date = new Date();
      const d = `0${date.getUTCDate()}`.slice(-2);
      const m = "JanFebMarAprMayJunJulAugSepOctNovDec".substr(
        date.getUTCMonth() * 3,
        3
      );
      const Y = date.getUTCFullYear();
      const H = `0${date.getUTCHours()}`.slice(-2);
      const M = `0${date.getUTCMinutes()}`.slice(-2);
      const S = `0${date.getUTCSeconds()}`.slice(-2);
      return `${this.ip} - ${this.userId ||
      "-"} [${d}/${m}/${Y}:${H}:${M}:${S} +0000] ${JSON.stringify(
      `${this.method} ${this.path}${this.query ? `?${this.query}` : ""} ${
        this.protocol
      }`
    )} ${this.status} ${this.length} ${JSON.stringify(
      "Referer" in this.headers ? this.headers.Referer : "-"
    )} ${JSON.stringify(
      "User-Agent" in this.headers ? this.headers["User-Agent"] : "-"
    )}`;
    }
  }
  /** @type {(new () => any)[]} */ Context.watch = [];
  var Context_2 = Context;

  var ApiOperationStatus;
  (function (ApiOperationStatus) {
      ApiOperationStatus[ApiOperationStatus["SUCCESS"] = 0] = "SUCCESS";
      ApiOperationStatus[ApiOperationStatus["FAIL"] = 1] = "FAIL";
  })(ApiOperationStatus || (ApiOperationStatus = {}));
  var ApiEvent;
  (function (ApiEvent) {
      ApiEvent["UPLOAD"] = "UPLOAD";
      ApiEvent["UPLOAD_FINISHED"] = "UPLOAD_FINISHED";
      ApiEvent["DOWNLOAD"] = "DOWNLOAD";
      ApiEvent["DOWNLOAD_FINISHED"] = "DOWNLOAD_FINISHED";
  })(ApiEvent || (ApiEvent = {}));
  var ApiProviderType;
  (function (ApiProviderType) {
      ApiProviderType[ApiProviderType["GITHUB"] = 0] = "GITHUB";
      ApiProviderType[ApiProviderType["GITLAB"] = 1] = "GITLAB";
  })(ApiProviderType || (ApiProviderType = {}));

  class Github {
      constructor(gistId, userToken) {
          this.gistId = gistId;
          this.userToken = userToken;
      }
      async upload(syncData) {
          const files = Object.keys(syncData).reduce((acc, key) => {
              return Object.assign(Object.assign({}, acc), { [key]: {
                      content: JSON.stringify(syncData[key]),
                  } });
          }, {});
          const { status } = await Context_2.fetch(`${Github.GIST_API_URL}/${this.gistId}`, {
              headers: {
                  'User-Agent': 'Mozilla/5.0',
                  Authorization: `token ${this.userToken}`,
              },
              body: {
                  description: 'Extensions Sync',
                  files,
              },
              method: 'PATCH',
          });
          return status === 200 ? ApiOperationStatus.SUCCESS : ApiOperationStatus.FAIL;
      }
      async download() {
          const { body } = await Context_2.fetch(`${Github.GIST_API_URL}/${this.gistId}`, {
              headers: {
                  'User-Agent': 'Mozilla/5.0',
                  Authorization: `token ${this.userToken}`,
              },
              method: 'GET',
          });
          const syncData = Object.keys(body.files).reduce((acc, key) => {
              return Object.assign(Object.assign({}, acc), { [key]: JSON.parse(body.files[key].content) });
          }, { extensions: {}, keybindings: {}, tweaks: {} });
          return syncData;
      }
      getName() {
          return 'Github';
      }
  }
  Github.GIST_API_URL = 'https://api.github.com/gists';

  class Gitlab {
      constructor(snippetId, userToken) {
          this.snippetId = snippetId;
          this.userToken = userToken;
      }
      async upload(syncData) {
          const { status } = await Context_2.fetch(`${Gitlab.SNIPPETS_API_URL}/${this.snippetId}`, {
              headers: {
                  'User-Agent': 'Mozilla/5.0',
                  'PRIVATE-TOKEN': `${this.userToken}`,
                  'Content-Type': 'application/json',
              },
              body: {
                  title: 'Extensions Sync',
                  content: JSON.stringify(syncData),
              },
              method: 'PUT',
          });
          return status === 200 ? ApiOperationStatus.SUCCESS : ApiOperationStatus.FAIL;
      }
      async download() {
          const { body } = await Context_2.fetch(`${Gitlab.SNIPPETS_API_URL}/${this.snippetId}/raw`, {
              headers: {
                  'User-Agent': 'Mozilla/5.0',
                  'PRIVATE-TOKEN': `${this.userToken}`,
              },
              method: 'GET',
          });
          return body;
      }
      getName() {
          return 'Gitlab';
      }
  }
  Gitlab.SNIPPETS_API_URL = 'https://gitlab.com/api/v4/snippets';

  const logger = (prefix) => (content) => log(`[extensions-sync] [${prefix}] ${content}`);
  const execute = async (command) => {
      const process = new Gio2_0.Subprocess({
          argv: ['bash', '-c', command],
          flags: Gio2_0.SubprocessFlags.STDOUT_PIPE,
      });
      process.init(null);
      return new Promise((resolve, reject) => {
          process.communicate_utf8_async(null, null, (_, result) => {
              const [, stdout, stderr] = process.communicate_utf8_finish(result);
              if (stderr) {
                  reject(stderr);
              }
              else if (stdout) {
                  resolve(stdout.trim());
              }
              else {
                  resolve();
              }
          });
      });
  };
  const settingsFlagsToEnumList = (flags) => flags
      .toString(2)
      .split('')
      .reverse()
      .map((state) => parseInt(state, 10))
      .map((state, index) => {
      if (state === 1) {
          return index;
      }
  })
      .filter((value) => value !== undefined);

  const debug = logger('shell');
  var ExtensionType;
  (function (ExtensionType) {
      ExtensionType[ExtensionType["SYSTEM"] = 1] = "SYSTEM";
      ExtensionType[ExtensionType["PER_USER"] = 2] = "PER_USER";
  })(ExtensionType || (ExtensionType = {}));
  var ExtensionState;
  (function (ExtensionState) {
      ExtensionState[ExtensionState["ENABLED"] = 1] = "ENABLED";
      ExtensionState[ExtensionState["DISABLED"] = 2] = "DISABLED";
      ExtensionState[ExtensionState["ERROR"] = 3] = "ERROR";
      ExtensionState[ExtensionState["OUT_OF_DATE"] = 4] = "OUT_OF_DATE";
      ExtensionState[ExtensionState["DOWNLOADING"] = 5] = "DOWNLOADING";
      ExtensionState[ExtensionState["INITIALIZED"] = 6] = "INITIALIZED";
      // Used as an error state for operations on unknown extensions,
      // should never be in a real extensionMeta object.
      ExtensionState[ExtensionState["UNINSTALLED"] = 99] = "UNINSTALLED";
  })(ExtensionState || (ExtensionState = {}));
  const getCurrentExtension = () => imports.misc.extensionUtils.getCurrentExtension();
  const getCurrentExtensionSettings = () => imports.misc.extensionUtils.getSettings();
  const canRestartShell = () => {
      return !imports.gi.Meta.is_wayland_compositor();
  };
  const restartShell = (text) => {
      if (!imports.gi.Meta.is_wayland_compositor()) {
          imports.gi.Meta.restart(text);
      }
  };
  const notify = (text) => imports.ui.main.notify(text);
  const writeDconfData = async (schemaPath, data) => {
      if (!schemaPath || !data) {
          return;
      }
      const [file, ioStream] = Gio2_0.file_new_tmp(null);
      file.replace_contents(imports.byteArray.fromString(data), null, false, Gio2_0.FileCreateFlags.REPLACE_DESTINATION, null);
      try {
          await execute(`dconf load ${schemaPath} < ${file.get_path()}`);
          debug(`loaded settings for ${schemaPath}`);
      }
      catch (ex) {
          debug(`cannot load settings for ${schemaPath}`);
      }
      file.delete(null);
      ioStream.close_async(GLib2_0.PRIORITY_DEFAULT, null, null);
  };
  const readDconfData = async (schemaPath) => {
      return execute(`dconf dump ${schemaPath}`);
  };

  const debug$1 = logger('api');
  class Api {
      constructor(eventEmitter, data) {
          this.data = data;
          this.settings = getCurrentExtensionSettings();
          this.provider = this.createProvider();
          this.eventEmitter = eventEmitter;
          this.eventEmitter.on(ApiEvent.UPLOAD, this.upload.bind(this));
          this.eventEmitter.on(ApiEvent.DOWNLOAD, this.download.bind(this));
          this.settings.connect('changed', this.updateProvider.bind(this));
      }
      async upload() {
          debug$1('got upload request, uploading settings...');
          try {
              const status = await this.provider.upload(Object.assign({}, (await this.data.getSyncData())));
              if (status === ApiOperationStatus.FAIL) {
                  throw new Error('Could not upload');
              }
              debug$1(`uploaded settings to ${this.provider.getName()} successfully`);
              this.eventEmitter.emit(ApiEvent.UPLOAD_FINISHED, status);
              notify(_(`Settings successfully uploaded to ${this.provider.getName()}`));
          }
          catch (ex) {
              this.eventEmitter.emit(ApiEvent.UPLOAD_FINISHED, undefined, ex);
              notify(_(`Error occured while uploading settings to ${this.provider.getName()}. Please check the logs.`));
              debug$1(`error occured during upload ${ex}`);
          }
      }
      async download() {
          debug$1('got download request, downloading settings...');
          try {
              const result = await this.provider.download();
              debug$1(`downloaded settings from ${this.provider.getName()} successfully`);
              this.eventEmitter.emit(ApiEvent.DOWNLOAD_FINISHED, result);
          }
          catch (ex) {
              this.eventEmitter.emit(ApiEvent.DOWNLOAD_FINISHED, undefined, ex);
              notify(_(`Error occured while downloading settings from ${this.provider.getName()}. Please check the logs.`));
              debug$1(`error occured during download ${ex}`);
          }
      }
      createProvider() {
          const providerType = this.settings.get_enum('provider');
          debug$1(`changing provider to ${ApiProviderType[providerType]}`);
          switch (providerType) {
              case ApiProviderType.GITHUB:
                  return this.createGithubProvider();
              case ApiProviderType.GITLAB:
                  return this.createGitlabProvider();
              default:
                  return this.createGithubProvider();
          }
      }
      updateProvider() {
          this.provider = this.createProvider();
      }
      createGithubProvider() {
          const gistId = this.settings.get_string('github-gist-id');
          const userToken = this.settings.get_string('github-user-token');
          return new Github(gistId, userToken);
      }
      createGitlabProvider() {
          const snippetId = this.settings.get_string('gitlab-snippet-id');
          const userToken = this.settings.get_string('gitlab-user-token');
          return new Gitlab(snippetId, userToken);
      }
  }

  function createCommonjsModule(fn, basedir, module) {
  	return module = {
  	  path: basedir,
  	  exports: {},
  	  require: function (path, base) {
        return commonjsRequire(path, (base === undefined || base === null) ? module.path : base);
      }
  	}, fn(module, module.exports), module.exports;
  }

  function commonjsRequire () {
  	throw new Error('Dynamic requires are not currently supported by @rollup/plugin-commonjs');
  }

  var util = createCommonjsModule(function (module, exports) {

  const nameStartChar = ':A-Za-z_\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD';
  const nameChar = nameStartChar + '\\-.\\d\\u00B7\\u0300-\\u036F\\u203F-\\u2040';
  const nameRegexp = '[' + nameStartChar + '][' + nameChar + ']*';
  const regexName = new RegExp('^' + nameRegexp + '$');

  const getAllMatches = function(string, regex) {
    const matches = [];
    let match = regex.exec(string);
    while (match) {
      const allmatches = [];
      const len = match.length;
      for (let index = 0; index < len; index++) {
        allmatches.push(match[index]);
      }
      matches.push(allmatches);
      match = regex.exec(string);
    }
    return matches;
  };

  const isName = function(string) {
    const match = regexName.exec(string);
    return !(match === null || typeof match === 'undefined');
  };

  exports.isExist = function(v) {
    return typeof v !== 'undefined';
  };

  exports.isEmptyObject = function(obj) {
    return Object.keys(obj).length === 0;
  };

  /**
   * Copy all the properties of a into b.
   * @param {*} target
   * @param {*} a
   */
  exports.merge = function(target, a, arrayMode) {
    if (a) {
      const keys = Object.keys(a); // will return an array of own properties
      const len = keys.length; //don't make it inline
      for (let i = 0; i < len; i++) {
        if(arrayMode === 'strict'){
          target[keys[i]] = [ a[keys[i]] ];
        }else {
          target[keys[i]] = a[keys[i]];
        }
      }
    }
  };
  /* exports.merge =function (b,a){
    return Object.assign(b,a);
  } */

  exports.getValue = function(v) {
    if (exports.isExist(v)) {
      return v;
    } else {
      return '';
    }
  };

  // const fakeCall = function(a) {return a;};
  // const fakeCallNoReturn = function() {};

  exports.buildOptions = function(options, defaultOptions, props) {
    var newOptions = {};
    if (!options) {
      return defaultOptions; //if there are not options
    }

    for (let i = 0; i < props.length; i++) {
      if (options[props[i]] !== undefined) {
        newOptions[props[i]] = options[props[i]];
      } else {
        newOptions[props[i]] = defaultOptions[props[i]];
      }
    }
    return newOptions;
  };

  exports.isName = isName;
  exports.getAllMatches = getAllMatches;
  exports.nameRegexp = nameRegexp;
  });

  const convertToJson = function(node, options) {
    const jObj = {};

    //when no child node or attr is present
    if ((!node.child || util.isEmptyObject(node.child)) && (!node.attrsMap || util.isEmptyObject(node.attrsMap))) {
      return util.isExist(node.val) ? node.val : '';
    } else {
      //otherwise create a textnode if node has some text
      if (util.isExist(node.val)) {
        if (!(typeof node.val === 'string' && (node.val === '' || node.val === options.cdataPositionChar))) {
          if(options.arrayMode === "strict"){
            jObj[options.textNodeName] = [ node.val ];
          }else {
            jObj[options.textNodeName] = node.val;
          }
        }
      }
    }

    util.merge(jObj, node.attrsMap, options.arrayMode);

    const keys = Object.keys(node.child);
    for (let index = 0; index < keys.length; index++) {
      var tagname = keys[index];
      if (node.child[tagname] && node.child[tagname].length > 1) {
        jObj[tagname] = [];
        for (var tag in node.child[tagname]) {
          jObj[tagname].push(convertToJson(node.child[tagname][tag], options));
        }
      } else {
        if(options.arrayMode === true){
          const result = convertToJson(node.child[tagname][0], options);
          if(typeof result === 'object')
            jObj[tagname] = [ result ];
          else
            jObj[tagname] = result;
        }else if(options.arrayMode === "strict"){
          jObj[tagname] = [convertToJson(node.child[tagname][0], options) ];
        }else {
          jObj[tagname] = convertToJson(node.child[tagname][0], options);
        }
      }
    }

    //add value
    return jObj;
  };

  var convertToJson_1 = convertToJson;

  var node2json = {
  	convertToJson: convertToJson_1
  };

  var xmlNode = function(tagname, parent, val) {
    this.tagname = tagname;
    this.parent = parent;
    this.child = {}; //child tags
    this.attrsMap = {}; //attributes map
    this.val = val; //text only
    this.addChild = function(child) {
      if (Array.isArray(this.child[child.tagname])) {
        //already presents
        this.child[child.tagname].push(child);
      } else {
        this.child[child.tagname] = [child];
      }
    };
  };

  const buildOptions = util.buildOptions;

  const regx =
    '<((!\\[CDATA\\[([\\s\\S]*?)(]]>))|((NAME:)?(NAME))([^>]*)>|((\\/)(NAME)\\s*>))([^<]*)'
    .replace(/NAME/g, util.nameRegexp);

  //const tagsRegx = new RegExp("<(\\/?[\\w:\\-\._]+)([^>]*)>(\\s*"+cdataRegx+")*([^<]+)?","g");
  //const tagsRegx = new RegExp("<(\\/?)((\\w*:)?([\\w:\\-\._]+))([^>]*)>([^<]*)("+cdataRegx+"([^<]*))*([^<]+)?","g");

  //polyfill
  if (!Number.parseInt && window.parseInt) {
    Number.parseInt = window.parseInt;
  }
  if (!Number.parseFloat && window.parseFloat) {
    Number.parseFloat = window.parseFloat;
  }

  const defaultOptions = {
    attributeNamePrefix: '@_',
    attrNodeName: false,
    textNodeName: '#text',
    ignoreAttributes: true,
    ignoreNameSpace: false,
    allowBooleanAttributes: false, //a tag can have attributes without any value
    //ignoreRootElement : false,
    parseNodeValue: true,
    parseAttributeValue: false,
    arrayMode: false,
    trimValues: true, //Trim string values of tag and attributes
    cdataTagName: false,
    cdataPositionChar: '\\c',
    tagValueProcessor: function(a, tagName) {
      return a;
    },
    attrValueProcessor: function(a, attrName) {
      return a;
    },
    stopNodes: []
    //decodeStrict: false,
  };

  var defaultOptions_1 = defaultOptions;

  const props = [
    'attributeNamePrefix',
    'attrNodeName',
    'textNodeName',
    'ignoreAttributes',
    'ignoreNameSpace',
    'allowBooleanAttributes',
    'parseNodeValue',
    'parseAttributeValue',
    'arrayMode',
    'trimValues',
    'cdataTagName',
    'cdataPositionChar',
    'tagValueProcessor',
    'attrValueProcessor',
    'parseTrueNumberOnly',
    'stopNodes'
  ];
  var props_1 = props;

  /**
   * Trim -> valueProcessor -> parse value
   * @param {string} tagName
   * @param {string} val
   * @param {object} options
   */
  function processTagValue(tagName, val, options) {
    if (val) {
      if (options.trimValues) {
        val = val.trim();
      }
      val = options.tagValueProcessor(val, tagName);
      val = parseValue(val, options.parseNodeValue, options.parseTrueNumberOnly);
    }

    return val;
  }

  function resolveNameSpace(tagname, options) {
    if (options.ignoreNameSpace) {
      const tags = tagname.split(':');
      const prefix = tagname.charAt(0) === '/' ? '/' : '';
      if (tags[0] === 'xmlns') {
        return '';
      }
      if (tags.length === 2) {
        tagname = prefix + tags[1];
      }
    }
    return tagname;
  }

  function parseValue(val, shouldParse, parseTrueNumberOnly) {
    if (shouldParse && typeof val === 'string') {
      let parsed;
      if (val.trim() === '' || isNaN(val)) {
        parsed = val === 'true' ? true : val === 'false' ? false : val;
      } else {
        if (val.indexOf('0x') !== -1) {
          //support hexa decimal
          parsed = Number.parseInt(val, 16);
        } else if (val.indexOf('.') !== -1) {
          parsed = Number.parseFloat(val);
          val = val.replace(/\.?0+$/, "");
        } else {
          parsed = Number.parseInt(val, 10);
        }
        if (parseTrueNumberOnly) {
          parsed = String(parsed) === val ? parsed : val;
        }
      }
      return parsed;
    } else {
      if (util.isExist(val)) {
        return val;
      } else {
        return '';
      }
    }
  }

  //TODO: change regex to capture NS
  //const attrsRegx = new RegExp("([\\w\\-\\.\\:]+)\\s*=\\s*(['\"])((.|\n)*?)\\2","gm");
  const attrsRegx = new RegExp('([^\\s=]+)\\s*(=\\s*([\'"])(.*?)\\3)?', 'g');

  function buildAttributesMap(attrStr, options) {
    if (!options.ignoreAttributes && typeof attrStr === 'string') {
      attrStr = attrStr.replace(/\r?\n/g, ' ');
      //attrStr = attrStr || attrStr.trim();

      const matches = util.getAllMatches(attrStr, attrsRegx);
      const len = matches.length; //don't make it inline
      const attrs = {};
      for (let i = 0; i < len; i++) {
        const attrName = resolveNameSpace(matches[i][1], options);
        if (attrName.length) {
          if (matches[i][4] !== undefined) {
            if (options.trimValues) {
              matches[i][4] = matches[i][4].trim();
            }
            matches[i][4] = options.attrValueProcessor(matches[i][4], attrName);
            attrs[options.attributeNamePrefix + attrName] = parseValue(
              matches[i][4],
              options.parseAttributeValue,
              options.parseTrueNumberOnly
            );
          } else if (options.allowBooleanAttributes) {
            attrs[options.attributeNamePrefix + attrName] = true;
          }
        }
      }
      if (!Object.keys(attrs).length) {
        return;
      }
      if (options.attrNodeName) {
        const attrCollection = {};
        attrCollection[options.attrNodeName] = attrs;
        return attrCollection;
      }
      return attrs;
    }
  }

  const getTraversalObj = function(xmlData, options) {
    xmlData = xmlData.replace(/(\r\n)|\n/, " ");
    options = buildOptions(options, defaultOptions, props);
    const xmlObj = new xmlNode('!xml');
    let currentNode = xmlObj;
    let textData = "";

  //function match(xmlData){
    for(let i=0; i< xmlData.length; i++){
      const ch = xmlData[i];
      if(ch === '<'){
        if( xmlData[i+1] === '/') {//Closing Tag
          const closeIndex = findClosingIndex(xmlData, ">", i, "Closing Tag is not closed.");
          let tagName = xmlData.substring(i+2,closeIndex).trim();

          if(options.ignoreNameSpace){
            const colonIndex = tagName.indexOf(":");
            if(colonIndex !== -1){
              tagName = tagName.substr(colonIndex+1);
            }
          }

          /* if (currentNode.parent) {
            currentNode.parent.val = util.getValue(currentNode.parent.val) + '' + processTagValue2(tagName, textData , options);
          } */
          if(currentNode){
            if(currentNode.val){
              currentNode.val = util.getValue(currentNode.val) + '' + processTagValue(tagName, textData , options);
            }else {
              currentNode.val = processTagValue(tagName, textData , options);
            }
          }

          if (options.stopNodes.length && options.stopNodes.includes(currentNode.tagname)) {
            currentNode.child = [];
            if (currentNode.attrsMap == undefined) { currentNode.attrsMap = {};}
            currentNode.val = xmlData.substr(currentNode.startIndex + 1, i - currentNode.startIndex - 1);
          }
          currentNode = currentNode.parent;
          textData = "";
          i = closeIndex;
        } else if( xmlData[i+1] === '?') {
          i = findClosingIndex(xmlData, "?>", i, "Pi Tag is not closed.");
        } else if(xmlData.substr(i + 1, 3) === '!--') {
          i = findClosingIndex(xmlData, "-->", i, "Comment is not closed.");
        } else if( xmlData.substr(i + 1, 2) === '!D') {
          const closeIndex = findClosingIndex(xmlData, ">", i, "DOCTYPE is not closed.");
          const tagExp = xmlData.substring(i, closeIndex);
          if(tagExp.indexOf("[") >= 0){
            i = xmlData.indexOf("]>", i) + 1;
          }else {
            i = closeIndex;
          }
        }else if(xmlData.substr(i + 1, 2) === '![') {
          const closeIndex = findClosingIndex(xmlData, "]]>", i, "CDATA is not closed.") - 2;
          const tagExp = xmlData.substring(i + 9,closeIndex);

          //considerations
          //1. CDATA will always have parent node
          //2. A tag with CDATA is not a leaf node so it's value would be string type.
          if(textData){
            currentNode.val = util.getValue(currentNode.val) + '' + processTagValue(currentNode.tagname, textData , options);
            textData = "";
          }

          if (options.cdataTagName) {
            //add cdata node
            const childNode = new xmlNode(options.cdataTagName, currentNode, tagExp);
            currentNode.addChild(childNode);
            //for backtracking
            currentNode.val = util.getValue(currentNode.val) + options.cdataPositionChar;
            //add rest value to parent node
            if (tagExp) {
              childNode.val = tagExp;
            }
          } else {
            currentNode.val = (currentNode.val || '') + (tagExp || '');
          }

          i = closeIndex + 2;
        }else {//Opening tag
          const result = closingIndexForOpeningTag(xmlData, i+1);
          let tagExp = result.data;
          const closeIndex = result.index;
          const separatorIndex = tagExp.indexOf(" ");
          let tagName = tagExp;
          if(separatorIndex !== -1){
            tagName = tagExp.substr(0, separatorIndex).trimRight();
            tagExp = tagExp.substr(separatorIndex + 1);
          }

          if(options.ignoreNameSpace){
            const colonIndex = tagName.indexOf(":");
            if(colonIndex !== -1){
              tagName = tagName.substr(colonIndex+1);
            }
          }

          //save text to parent node
          if (currentNode && textData) {
            if(currentNode.tagname !== '!xml'){
              currentNode.val = util.getValue(currentNode.val) + '' + processTagValue( currentNode.tagname, textData, options);
            }
          }

          if(tagExp.length > 0 && tagExp.lastIndexOf("/") === tagExp.length - 1){//selfClosing tag

            if(tagName[tagName.length - 1] === "/"){ //remove trailing '/'
              tagName = tagName.substr(0, tagName.length - 1);
              tagExp = tagName;
            }else {
              tagExp = tagExp.substr(0, tagExp.length - 1);
            }

            const childNode = new xmlNode(tagName, currentNode, '');
            if(tagName !== tagExp){
              childNode.attrsMap = buildAttributesMap(tagExp, options);
            }
            currentNode.addChild(childNode);
          }else {//opening tag

            const childNode = new xmlNode( tagName, currentNode );
            if (options.stopNodes.length && options.stopNodes.includes(childNode.tagname)) {
              childNode.startIndex=closeIndex;
            }
            if(tagName !== tagExp){
              childNode.attrsMap = buildAttributesMap(tagExp, options);
            }
            currentNode.addChild(childNode);
            currentNode = childNode;
          }
          textData = "";
          i = closeIndex;
        }
      }else {
        textData += xmlData[i];
      }
    }
    return xmlObj;
  };

  function closingIndexForOpeningTag(data, i){
    let attrBoundary;
    let tagExp = "";
    for (let index = i; index < data.length; index++) {
      let ch = data[index];
      if (attrBoundary) {
          if (ch === attrBoundary) attrBoundary = "";//reset
      } else if (ch === '"' || ch === "'") {
          attrBoundary = ch;
      } else if (ch === '>') {
          return {
            data: tagExp,
            index: index
          }
      } else if (ch === '\t') {
        ch = " ";
      }
      tagExp += ch;
    }
  }

  function findClosingIndex(xmlData, str, i, errMsg){
    const closingIndex = xmlData.indexOf(str, i);
    if(closingIndex === -1){
      throw new Error(errMsg)
    }else {
      return closingIndex + str.length - 1;
    }
  }

  var getTraversalObj_1 = getTraversalObj;

  var xmlstr2xmlnode = {
  	defaultOptions: defaultOptions_1,
  	props: props_1,
  	getTraversalObj: getTraversalObj_1
  };

  const defaultOptions$1 = {
    allowBooleanAttributes: false, //A tag can have attributes without any value
  };

  const props$1 = ['allowBooleanAttributes'];

  //const tagsPattern = new RegExp("<\\/?([\\w:\\-_\.]+)\\s*\/?>","g");
  var validate = function (xmlData, options) {
    options = util.buildOptions(options, defaultOptions$1, props$1);

    //xmlData = xmlData.replace(/(\r\n|\n|\r)/gm,"");//make it single line
    //xmlData = xmlData.replace(/(^\s*<\?xml.*?\?>)/g,"");//Remove XML starting tag
    //xmlData = xmlData.replace(/(<!DOCTYPE[\s\w\"\.\/\-\:]+(\[.*\])*\s*>)/g,"");//Remove DOCTYPE
    const tags = [];
    let tagFound = false;

    //indicates that the root tag has been closed (aka. depth 0 has been reached)
    let reachedRoot = false;

    if (xmlData[0] === '\ufeff') {
      // check for byte order mark (BOM)
      xmlData = xmlData.substr(1);
    }

    for (let i = 0; i < xmlData.length; i++) {
      if (xmlData[i] === '<') {
        //starting of tag
        //read until you reach to '>' avoiding any '>' in attribute value

        i++;
        if (xmlData[i] === '?') {
          i = readPI(xmlData, ++i);
          if (i.err) {
            return i;
          }
        } else if (xmlData[i] === '!') {
          i = readCommentAndCDATA(xmlData, i);
          continue;
        } else {
          let closingTag = false;
          if (xmlData[i] === '/') {
            //closing tag
            closingTag = true;
            i++;
          }
          //read tagname
          let tagName = '';
          for (; i < xmlData.length &&
            xmlData[i] !== '>' &&
            xmlData[i] !== ' ' &&
            xmlData[i] !== '\t' &&
            xmlData[i] !== '\n' &&
            xmlData[i] !== '\r'; i++
          ) {
            tagName += xmlData[i];
          }
          tagName = tagName.trim();
          //console.log(tagName);

          if (tagName[tagName.length - 1] === '/') {
            //self closing tag without attributes
            tagName = tagName.substring(0, tagName.length - 1);
            //continue;
            i--;
          }
          if (!validateTagName(tagName)) {
            let msg;
            if (tagName.trim().length === 0) {
              msg = "There is an unnecessary space between tag name and backward slash '</ ..'.";
            } else {
              msg = "Tag '"+tagName+"' is an invalid name.";
            }
            return getErrorObject('InvalidTag', msg, getLineNumberForPosition(xmlData, i));
          }

          const result = readAttributeStr(xmlData, i);
          if (result === false) {
            return getErrorObject('InvalidAttr', "Attributes for '"+tagName+"' have open quote.", getLineNumberForPosition(xmlData, i));
          }
          let attrStr = result.value;
          i = result.index;

          if (attrStr[attrStr.length - 1] === '/') {
            //self closing tag
            attrStr = attrStr.substring(0, attrStr.length - 1);
            const isValid = validateAttributeString(attrStr, options);
            if (isValid === true) {
              tagFound = true;
              //continue; //text may presents after self closing tag
            } else {
              //the result from the nested function returns the position of the error within the attribute
              //in order to get the 'true' error line, we need to calculate the position where the attribute begins (i - attrStr.length) and then add the position within the attribute
              //this gives us the absolute index in the entire xml, which we can use to find the line at last
              return getErrorObject(isValid.err.code, isValid.err.msg, getLineNumberForPosition(xmlData, i - attrStr.length + isValid.err.line));
            }
          } else if (closingTag) {
            if (!result.tagClosed) {
              return getErrorObject('InvalidTag', "Closing tag '"+tagName+"' doesn't have proper closing.", getLineNumberForPosition(xmlData, i));
            } else if (attrStr.trim().length > 0) {
              return getErrorObject('InvalidTag', "Closing tag '"+tagName+"' can't have attributes or invalid starting.", getLineNumberForPosition(xmlData, i));
            } else {
              const otg = tags.pop();
              if (tagName !== otg) {
                return getErrorObject('InvalidTag', "Closing tag '"+otg+"' is expected inplace of '"+tagName+"'.", getLineNumberForPosition(xmlData, i));
              }

              //when there are no more tags, we reached the root level.
              if (tags.length == 0) {
                reachedRoot = true;
              }
            }
          } else {
            const isValid = validateAttributeString(attrStr, options);
            if (isValid !== true) {
              //the result from the nested function returns the position of the error within the attribute
              //in order to get the 'true' error line, we need to calculate the position where the attribute begins (i - attrStr.length) and then add the position within the attribute
              //this gives us the absolute index in the entire xml, which we can use to find the line at last
              return getErrorObject(isValid.err.code, isValid.err.msg, getLineNumberForPosition(xmlData, i - attrStr.length + isValid.err.line));
            }

            //if the root level has been reached before ...
            if (reachedRoot === true) {
              return getErrorObject('InvalidXml', 'Multiple possible root nodes found.', getLineNumberForPosition(xmlData, i));
            } else {
              tags.push(tagName);
            }
            tagFound = true;
          }

          //skip tag text value
          //It may include comments and CDATA value
          for (i++; i < xmlData.length; i++) {
            if (xmlData[i] === '<') {
              if (xmlData[i + 1] === '!') {
                //comment or CADATA
                i++;
                i = readCommentAndCDATA(xmlData, i);
                continue;
              } else {
                break;
              }
            } else if (xmlData[i] === '&') {
              const afterAmp = validateAmpersand(xmlData, i);
              if (afterAmp == -1)
                return getErrorObject('InvalidChar', "char '&' is not expected.", getLineNumberForPosition(xmlData, i));
              i = afterAmp;
            }
          } //end of reading tag text value
          if (xmlData[i] === '<') {
            i--;
          }
        }
      } else {
        if (xmlData[i] === ' ' || xmlData[i] === '\t' || xmlData[i] === '\n' || xmlData[i] === '\r') {
          continue;
        }
        return getErrorObject('InvalidChar', "char '"+xmlData[i]+"' is not expected.", getLineNumberForPosition(xmlData, i));
      }
    }

    if (!tagFound) {
      return getErrorObject('InvalidXml', 'Start tag expected.', 1);
    } else if (tags.length > 0) {
      return getErrorObject('InvalidXml', "Invalid '"+JSON.stringify(tags, null, 4).replace(/\r?\n/g, '')+"' found.", 1);
    }

    return true;
  };

  /**
   * Read Processing insstructions and skip
   * @param {*} xmlData
   * @param {*} i
   */
  function readPI(xmlData, i) {
    var start = i;
    for (; i < xmlData.length; i++) {
      if (xmlData[i] == '?' || xmlData[i] == ' ') {
        //tagname
        var tagname = xmlData.substr(start, i - start);
        if (i > 5 && tagname === 'xml') {
          return getErrorObject('InvalidXml', 'XML declaration allowed only at the start of the document.', getLineNumberForPosition(xmlData, i));
        } else if (xmlData[i] == '?' && xmlData[i + 1] == '>') {
          //check if valid attribut string
          i++;
          break;
        } else {
          continue;
        }
      }
    }
    return i;
  }

  function readCommentAndCDATA(xmlData, i) {
    if (xmlData.length > i + 5 && xmlData[i + 1] === '-' && xmlData[i + 2] === '-') {
      //comment
      for (i += 3; i < xmlData.length; i++) {
        if (xmlData[i] === '-' && xmlData[i + 1] === '-' && xmlData[i + 2] === '>') {
          i += 2;
          break;
        }
      }
    } else if (
      xmlData.length > i + 8 &&
      xmlData[i + 1] === 'D' &&
      xmlData[i + 2] === 'O' &&
      xmlData[i + 3] === 'C' &&
      xmlData[i + 4] === 'T' &&
      xmlData[i + 5] === 'Y' &&
      xmlData[i + 6] === 'P' &&
      xmlData[i + 7] === 'E'
    ) {
      let angleBracketsCount = 1;
      for (i += 8; i < xmlData.length; i++) {
        if (xmlData[i] === '<') {
          angleBracketsCount++;
        } else if (xmlData[i] === '>') {
          angleBracketsCount--;
          if (angleBracketsCount === 0) {
            break;
          }
        }
      }
    } else if (
      xmlData.length > i + 9 &&
      xmlData[i + 1] === '[' &&
      xmlData[i + 2] === 'C' &&
      xmlData[i + 3] === 'D' &&
      xmlData[i + 4] === 'A' &&
      xmlData[i + 5] === 'T' &&
      xmlData[i + 6] === 'A' &&
      xmlData[i + 7] === '['
    ) {
      for (i += 8; i < xmlData.length; i++) {
        if (xmlData[i] === ']' && xmlData[i + 1] === ']' && xmlData[i + 2] === '>') {
          i += 2;
          break;
        }
      }
    }

    return i;
  }

  var doubleQuote = '"';
  var singleQuote = "'";

  /**
   * Keep reading xmlData until '<' is found outside the attribute value.
   * @param {string} xmlData
   * @param {number} i
   */
  function readAttributeStr(xmlData, i) {
    let attrStr = '';
    let startChar = '';
    let tagClosed = false;
    for (; i < xmlData.length; i++) {
      if (xmlData[i] === doubleQuote || xmlData[i] === singleQuote) {
        if (startChar === '') {
          startChar = xmlData[i];
        } else if (startChar !== xmlData[i]) {
          //if vaue is enclosed with double quote then single quotes are allowed inside the value and vice versa
          continue;
        } else {
          startChar = '';
        }
      } else if (xmlData[i] === '>') {
        if (startChar === '') {
          tagClosed = true;
          break;
        }
      }
      attrStr += xmlData[i];
    }
    if (startChar !== '') {
      return false;
    }

    return {
      value: attrStr,
      index: i,
      tagClosed: tagClosed
    };
  }

  /**
   * Select all the attributes whether valid or invalid.
   */
  const validAttrStrRegxp = new RegExp('(\\s*)([^\\s=]+)(\\s*=)?(\\s*([\'"])(([\\s\\S])*?)\\5)?', 'g');

  //attr, ="sd", a="amit's", a="sd"b="saf", ab  cd=""

  function validateAttributeString(attrStr, options) {
    //console.log("start:"+attrStr+":end");

    //if(attrStr.trim().length === 0) return true; //empty string

    const matches = util.getAllMatches(attrStr, validAttrStrRegxp);
    const attrNames = {};

    for (let i = 0; i < matches.length; i++) {
      if (matches[i][1].length === 0) {
        //nospace before attribute name: a="sd"b="saf"
        return getErrorObject('InvalidAttr', "Attribute '"+matches[i][2]+"' has no space in starting.", getPositionFromMatch(attrStr, matches[i][0]))
      } else if (matches[i][3] === undefined && !options.allowBooleanAttributes) {
        //independent attribute: ab
        return getErrorObject('InvalidAttr', "boolean attribute '"+matches[i][2]+"' is not allowed.", getPositionFromMatch(attrStr, matches[i][0]));
      }
      /* else if(matches[i][6] === undefined){//attribute without value: ab=
                      return { err: { code:"InvalidAttr",msg:"attribute " + matches[i][2] + " has no value assigned."}};
                  } */
      const attrName = matches[i][2];
      if (!validateAttrName(attrName)) {
        return getErrorObject('InvalidAttr', "Attribute '"+attrName+"' is an invalid name.", getPositionFromMatch(attrStr, matches[i][0]));
      }
      if (!attrNames.hasOwnProperty(attrName)) {
        //check for duplicate attribute.
        attrNames[attrName] = 1;
      } else {
        return getErrorObject('InvalidAttr', "Attribute '"+attrName+"' is repeated.", getPositionFromMatch(attrStr, matches[i][0]));
      }
    }

    return true;
  }

  function validateNumberAmpersand(xmlData, i) {
    let re = /\d/;
    if (xmlData[i] === 'x') {
      i++;
      re = /[\da-fA-F]/;
    }
    for (; i < xmlData.length; i++) {
      if (xmlData[i] === ';')
        return i;
      if (!xmlData[i].match(re))
        break;
    }
    return -1;
  }

  function validateAmpersand(xmlData, i) {
    // https://www.w3.org/TR/xml/#dt-charref
    i++;
    if (xmlData[i] === ';')
      return -1;
    if (xmlData[i] === '#') {
      i++;
      return validateNumberAmpersand(xmlData, i);
    }
    let count = 0;
    for (; i < xmlData.length; i++, count++) {
      if (xmlData[i].match(/\w/) && count < 20)
        continue;
      if (xmlData[i] === ';')
        break;
      return -1;
    }
    return i;
  }

  function getErrorObject(code, message, lineNumber) {
    return {
      err: {
        code: code,
        msg: message,
        line: lineNumber,
      },
    };
  }

  function validateAttrName(attrName) {
    return util.isName(attrName);
  }

  // const startsWithXML = /^xml/i;

  function validateTagName(tagname) {
    return util.isName(tagname) /* && !tagname.match(startsWithXML) */;
  }

  //this function returns the line number for the character at the given index
  function getLineNumberForPosition(xmlData, index) {
    var lines = xmlData.substring(0, index).split(/\r?\n/);
    return lines.length;
  }

  //this function returns the position of the last character of match within attrStr
  function getPositionFromMatch(attrStr, match) {
    return attrStr.indexOf(match) + match.length;
  }

  var validator = {
  	validate: validate
  };

  const char = function(a) {
    return String.fromCharCode(a);
  };

  const chars = {
    nilChar: char(176),
    missingChar: char(201),
    nilPremitive: char(175),
    missingPremitive: char(200),

    emptyChar: char(178),
    emptyValue: char(177), //empty Premitive

    boundryChar: char(179),

    objStart: char(198),
    arrStart: char(204),
    arrayEnd: char(185),
  };

  const charsArr = [
    chars.nilChar,
    chars.nilPremitive,
    chars.missingChar,
    chars.missingPremitive,
    chars.boundryChar,
    chars.emptyChar,
    chars.emptyValue,
    chars.arrayEnd,
    chars.objStart,
    chars.arrStart,
  ];

  const _e = function(node, e_schema, options) {
    if (typeof e_schema === 'string') {
      //premitive
      if (node && node[0] && node[0].val !== undefined) {
        return getValue(node[0].val);
      } else {
        return getValue(node);
      }
    } else {
      const hasValidData = hasData(node);
      if (hasValidData === true) {
        let str = '';
        if (Array.isArray(e_schema)) {
          //attributes can't be repeated. hence check in children tags only
          str += chars.arrStart;
          const itemSchema = e_schema[0];
          //var itemSchemaType = itemSchema;
          const arr_len = node.length;

          if (typeof itemSchema === 'string') {
            for (let arr_i = 0; arr_i < arr_len; arr_i++) {
              const r = getValue(node[arr_i].val);
              str = processValue(str, r);
            }
          } else {
            for (let arr_i = 0; arr_i < arr_len; arr_i++) {
              const r = _e(node[arr_i], itemSchema, options);
              str = processValue(str, r);
            }
          }
          str += chars.arrayEnd; //indicates that next item is not array item
        } else {
          //object
          str += chars.objStart;
          const keys = Object.keys(e_schema);
          if (Array.isArray(node)) {
            node = node[0];
          }
          for (let i in keys) {
            const key = keys[i];
            //a property defined in schema can be present either in attrsMap or children tags
            //options.textNodeName will not present in both maps, take it's value from val
            //options.attrNodeName will be present in attrsMap
            let r;
            if (!options.ignoreAttributes && node.attrsMap && node.attrsMap[key]) {
              r = _e(node.attrsMap[key], e_schema[key], options);
            } else if (key === options.textNodeName) {
              r = _e(node.val, e_schema[key], options);
            } else {
              r = _e(node.child[key], e_schema[key], options);
            }
            str = processValue(str, r);
          }
        }
        return str;
      } else {
        return hasValidData;
      }
    }
  };

  const getValue = function(a /*, type*/) {
    switch (a) {
      case undefined:
        return chars.missingPremitive;
      case null:
        return chars.nilPremitive;
      case '':
        return chars.emptyValue;
      default:
        return a;
    }
  };

  const processValue = function(str, r) {
    if (!isAppChar(r[0]) && !isAppChar(str[str.length - 1])) {
      str += chars.boundryChar;
    }
    return str + r;
  };

  const isAppChar = function(ch) {
    return charsArr.indexOf(ch) !== -1;
  };

  function hasData(jObj) {
    if (jObj === undefined) {
      return chars.missingChar;
    } else if (jObj === null) {
      return chars.nilChar;
    } else if (
      jObj.child &&
      Object.keys(jObj.child).length === 0 &&
      (!jObj.attrsMap || Object.keys(jObj.attrsMap).length === 0)
    ) {
      return chars.emptyChar;
    } else {
      return true;
    }
  }


  const buildOptions$1 = util.buildOptions;

  const convert2nimn = function(node, e_schema, options) {
    options = buildOptions$1(options, xmlstr2xmlnode.defaultOptions, xmlstr2xmlnode.props);
    return _e(node, e_schema, options);
  };

  var convert2nimn_1 = convert2nimn;

  var nimndata = {
  	convert2nimn: convert2nimn_1
  };

  const buildOptions$2 = util.buildOptions;


  //TODO: do it later
  const convertToJsonString = function(node, options) {
    options = buildOptions$2(options, xmlstr2xmlnode.defaultOptions, xmlstr2xmlnode.props);

    options.indentBy = options.indentBy || '';
    return _cToJsonStr(node, options);
  };

  const _cToJsonStr = function(node, options, level) {
    let jObj = '{';

    //traver through all the children
    const keys = Object.keys(node.child);

    for (let index = 0; index < keys.length; index++) {
      var tagname = keys[index];
      if (node.child[tagname] && node.child[tagname].length > 1) {
        jObj += '"' + tagname + '" : [ ';
        for (var tag in node.child[tagname]) {
          jObj += _cToJsonStr(node.child[tagname][tag], options) + ' , ';
        }
        jObj = jObj.substr(0, jObj.length - 1) + ' ] '; //remove extra comma in last
      } else {
        jObj += '"' + tagname + '" : ' + _cToJsonStr(node.child[tagname][0], options) + ' ,';
      }
    }
    util.merge(jObj, node.attrsMap);
    //add attrsMap as new children
    if (util.isEmptyObject(jObj)) {
      return util.isExist(node.val) ? node.val : '';
    } else {
      if (util.isExist(node.val)) {
        if (!(typeof node.val === 'string' && (node.val === '' || node.val === options.cdataPositionChar))) {
          jObj += '"' + options.textNodeName + '" : ' + stringval(node.val);
        }
      }
    }
    //add value
    if (jObj[jObj.length - 1] === ',') {
      jObj = jObj.substr(0, jObj.length - 2);
    }
    return jObj + '}';
  };

  function stringval(v) {
    if (v === true || v === false || !isNaN(v)) {
      return v;
    } else {
      return '"' + v + '"';
    }
  }

  var convertToJsonString_1 = convertToJsonString;

  var node2json_str = {
  	convertToJsonString: convertToJsonString_1
  };

  //parse Empty Node as self closing node
  const buildOptions$3 = util.buildOptions;

  const defaultOptions$2 = {
    attributeNamePrefix: '@_',
    attrNodeName: false,
    textNodeName: '#text',
    ignoreAttributes: true,
    cdataTagName: false,
    cdataPositionChar: '\\c',
    format: false,
    indentBy: '  ',
    supressEmptyNode: false,
    tagValueProcessor: function(a) {
      return a;
    },
    attrValueProcessor: function(a) {
      return a;
    },
  };

  const props$2 = [
    'attributeNamePrefix',
    'attrNodeName',
    'textNodeName',
    'ignoreAttributes',
    'cdataTagName',
    'cdataPositionChar',
    'format',
    'indentBy',
    'supressEmptyNode',
    'tagValueProcessor',
    'attrValueProcessor',
  ];

  function Parser(options) {
    this.options = buildOptions$3(options, defaultOptions$2, props$2);
    if (this.options.ignoreAttributes || this.options.attrNodeName) {
      this.isAttribute = function(/*a*/) {
        return false;
      };
    } else {
      this.attrPrefixLen = this.options.attributeNamePrefix.length;
      this.isAttribute = isAttribute;
    }
    if (this.options.cdataTagName) {
      this.isCDATA = isCDATA;
    } else {
      this.isCDATA = function(/*a*/) {
        return false;
      };
    }
    this.replaceCDATAstr = replaceCDATAstr;
    this.replaceCDATAarr = replaceCDATAarr;

    if (this.options.format) {
      this.indentate = indentate;
      this.tagEndChar = '>\n';
      this.newLine = '\n';
    } else {
      this.indentate = function() {
        return '';
      };
      this.tagEndChar = '>';
      this.newLine = '';
    }

    if (this.options.supressEmptyNode) {
      this.buildTextNode = buildEmptyTextNode;
      this.buildObjNode = buildEmptyObjNode;
    } else {
      this.buildTextNode = buildTextValNode;
      this.buildObjNode = buildObjectNode;
    }

    this.buildTextValNode = buildTextValNode;
    this.buildObjectNode = buildObjectNode;
  }

  Parser.prototype.parse = function(jObj) {
    return this.j2x(jObj, 0).val;
  };

  Parser.prototype.j2x = function(jObj, level) {
    let attrStr = '';
    let val = '';
    const keys = Object.keys(jObj);
    const len = keys.length;
    for (let i = 0; i < len; i++) {
      const key = keys[i];
      if (typeof jObj[key] === 'undefined') ; else if (jObj[key] === null) {
        val += this.indentate(level) + '<' + key + '/' + this.tagEndChar;
      } else if (jObj[key] instanceof Date) {
        val += this.buildTextNode(jObj[key], key, '', level);
      } else if (typeof jObj[key] !== 'object') {
        //premitive type
        const attr = this.isAttribute(key);
        if (attr) {
          attrStr += ' ' + attr + '="' + this.options.attrValueProcessor('' + jObj[key]) + '"';
        } else if (this.isCDATA(key)) {
          if (jObj[this.options.textNodeName]) {
            val += this.replaceCDATAstr(jObj[this.options.textNodeName], jObj[key]);
          } else {
            val += this.replaceCDATAstr('', jObj[key]);
          }
        } else {
          //tag value
          if (key === this.options.textNodeName) {
            if (jObj[this.options.cdataTagName]) ; else {
              val += this.options.tagValueProcessor('' + jObj[key]);
            }
          } else {
            val += this.buildTextNode(jObj[key], key, '', level);
          }
        }
      } else if (Array.isArray(jObj[key])) {
        //repeated nodes
        if (this.isCDATA(key)) {
          val += this.indentate(level);
          if (jObj[this.options.textNodeName]) {
            val += this.replaceCDATAarr(jObj[this.options.textNodeName], jObj[key]);
          } else {
            val += this.replaceCDATAarr('', jObj[key]);
          }
        } else {
          //nested nodes
          const arrLen = jObj[key].length;
          for (let j = 0; j < arrLen; j++) {
            const item = jObj[key][j];
            if (typeof item === 'undefined') ; else if (item === null) {
              val += this.indentate(level) + '<' + key + '/' + this.tagEndChar;
            } else if (typeof item === 'object') {
              const result = this.j2x(item, level + 1);
              val += this.buildObjNode(result.val, key, result.attrStr, level);
            } else {
              val += this.buildTextNode(item, key, '', level);
            }
          }
        }
      } else {
        //nested node
        if (this.options.attrNodeName && key === this.options.attrNodeName) {
          const Ks = Object.keys(jObj[key]);
          const L = Ks.length;
          for (let j = 0; j < L; j++) {
            attrStr += ' ' + Ks[j] + '="' + this.options.attrValueProcessor('' + jObj[key][Ks[j]]) + '"';
          }
        } else {
          const result = this.j2x(jObj[key], level + 1);
          val += this.buildObjNode(result.val, key, result.attrStr, level);
        }
      }
    }
    return {attrStr: attrStr, val: val};
  };

  function replaceCDATAstr(str, cdata) {
    str = this.options.tagValueProcessor('' + str);
    if (this.options.cdataPositionChar === '' || str === '') {
      return str + '<![CDATA[' + cdata + ']]' + this.tagEndChar;
    } else {
      return str.replace(this.options.cdataPositionChar, '<![CDATA[' + cdata + ']]' + this.tagEndChar);
    }
  }

  function replaceCDATAarr(str, cdata) {
    str = this.options.tagValueProcessor('' + str);
    if (this.options.cdataPositionChar === '' || str === '') {
      return str + '<![CDATA[' + cdata.join(']]><![CDATA[') + ']]' + this.tagEndChar;
    } else {
      for (let v in cdata) {
        str = str.replace(this.options.cdataPositionChar, '<![CDATA[' + cdata[v] + ']]>');
      }
      return str + this.newLine;
    }
  }

  function buildObjectNode(val, key, attrStr, level) {
    if (attrStr && !val.includes('<')) {
      return (
        this.indentate(level) +
        '<' +
        key +
        attrStr +
        '>' +
        val +
        //+ this.newLine
        // + this.indentate(level)
        '</' +
        key +
        this.tagEndChar
      );
    } else {
      return (
        this.indentate(level) +
        '<' +
        key +
        attrStr +
        this.tagEndChar +
        val +
        //+ this.newLine
        this.indentate(level) +
        '</' +
        key +
        this.tagEndChar
      );
    }
  }

  function buildEmptyObjNode(val, key, attrStr, level) {
    if (val !== '') {
      return this.buildObjectNode(val, key, attrStr, level);
    } else {
      return this.indentate(level) + '<' + key + attrStr + '/' + this.tagEndChar;
      //+ this.newLine
    }
  }

  function buildTextValNode(val, key, attrStr, level) {
    return (
      this.indentate(level) +
      '<' +
      key +
      attrStr +
      '>' +
      this.options.tagValueProcessor(val) +
      '</' +
      key +
      this.tagEndChar
    );
  }

  function buildEmptyTextNode(val, key, attrStr, level) {
    if (val !== '') {
      return this.buildTextValNode(val, key, attrStr, level);
    } else {
      return this.indentate(level) + '<' + key + attrStr + '/' + this.tagEndChar;
    }
  }

  function indentate(level) {
    return this.options.indentBy.repeat(level);
  }

  function isAttribute(name /*, options*/) {
    if (name.startsWith(this.options.attributeNamePrefix)) {
      return name.substr(this.attrPrefixLen);
    } else {
      return false;
    }
  }

  function isCDATA(name) {
    return name === this.options.cdataTagName;
  }

  //formatting
  //indentation
  //\n after each closing or self closing tag

  var json2xml = Parser;

  var parser = createCommonjsModule(function (module, exports) {



  const x2xmlnode = xmlstr2xmlnode;
  const buildOptions = util.buildOptions;


  exports.parse = function(xmlData, options, validationOption) {
    if( validationOption){
      if(validationOption === true) validationOption = {};
      
      const result = validator.validate(xmlData, validationOption);
      if (result !== true) {
        throw Error( result.err.msg)
      }
    }
    options = buildOptions(options, x2xmlnode.defaultOptions, x2xmlnode.props);
    const traversableObj = xmlstr2xmlnode.getTraversalObj(xmlData, options);
    //print(traversableObj, "  ");
    return node2json.convertToJson(traversableObj, options);
  };
  exports.convertTonimn = nimndata.convert2nimn;
  exports.getTraversalObj = xmlstr2xmlnode.getTraversalObj;
  exports.convertToJson = node2json.convertToJson;
  exports.convertToJsonString = node2json_str.convertToJsonString;
  exports.validate = validator.validate;
  exports.j2xParser = json2xml;
  exports.parseToNimn = function(xmlData, schema, options) {
    return exports.convertTonimn(exports.getTraversalObj(xmlData, options), schema, options);
  };
  });

  const debug$2 = logger('extension-utils');
  const readSchemaAsJson = (schemaPath) => {
      const [, contents] = GLib2_0.file_get_contents(schemaPath);
      return parser.parse(imports.byteArray.toString(contents), { ignoreAttributes: false });
  };
  const getExtensionManager = () => imports.ui.main.extensionManager;
  const getExtensionById = (extensionId) => getExtensionManager().lookup(extensionId);
  const getExtensionSchemas = async (extensionId) => {
      const extension = getExtensionById(extensionId);
      let stdout;
      try {
          stdout = await execute(`find -L ${extension.path} -iname "*.xml" -exec grep -l "schemalist" {} +`);
      }
      catch (ex) {
          debug$2(`error occured while getting extension schemas: ${ex}`);
          return {};
      }
      if (!stdout) {
          return {};
      }
      const schemaFiles = stdout.split('\n');
      const foundSchemas = schemaFiles
          .map((schemaFile) => readSchemaAsJson(schemaFile))
          .reduce((schemaJsonAcc, schemaJson) => {
          if (!schemaJson || !schemaJson.schemalist || !schemaJson.schemalist.schema) {
              return schemaJsonAcc;
          }
          const schema = schemaJson.schemalist.schema;
          if (Array.isArray(schema)) {
              const multipleSchemaObj = schema.reduce((acc, schemaObj) => {
                  if (schemaObj['@_path']) {
                      return Object.assign(Object.assign({}, acc), { [schemaObj['@_path']]: {} });
                  }
                  return acc;
              }, {});
              return Object.assign(Object.assign({}, multipleSchemaObj), schemaJsonAcc);
          }
          else if (schema['@_path']) {
              return Object.assign(Object.assign({}, schemaJsonAcc), { [schema['@_path']]: {} });
          }
          return schemaJsonAcc;
      }, {});
      return foundSchemas;
  };
  const getExtensionIds = () => getExtensionManager()
      .getUuids()
      .filter((uuid) => getExtensionById(uuid).type === ExtensionType.PER_USER && uuid !== getCurrentExtension().metadata.uuid);
  const getAllExtensions = () => {
      const extensionIds = getExtensionIds();
      const extensions = extensionIds
          .map((id) => {
          const extension = getExtensionById(id);
          if (extension.type === ExtensionType.PER_USER) {
              return extension;
          }
          return undefined;
      })
          .filter((item) => item !== undefined);
      return extensions;
  };
  const getExtensionConfigData = async (extensionId) => {
      const schemas = await getExtensionSchemas(extensionId);
      return Object.keys(schemas).reduce(async (acc, schema) => {
          try {
              return Object.assign(Object.assign({}, (await acc)), { [schema]: await readDconfData(schema) });
          }
          catch (ex) {
              debug$2(`cannot dump settings for ${extensionId}:${schema}`);
          }
          return acc;
      }, Promise.resolve({}));
  };
  const getAllExtensionConfigData = async () => {
      const extensions = getAllExtensions();
      return extensions.reduce(async (extensionAcc, extension) => {
          return Object.assign(Object.assign({}, (await extensionAcc)), { [extension.metadata.uuid]: await getExtensionConfigData(extension.metadata.uuid) });
      }, Promise.resolve({}));
  };
  const removeExtension = (extensionId) => {
      imports.ui.extensionDownloader.uninstallExtension(extensionId);
      debug$2(`removed extension ${extensionId}`);
  };
  const gotExtensionZipFile = (session, message, uuid, dir, callback, errback) => {
      if (message.status_code != Soup2_4.KnownStatusCode.OK) {
          errback('DownloadExtensionError', message.status_code);
          return;
      }
      try {
          if (!dir.query_exists(null))
              dir.make_directory_with_parents(null);
      }
      catch (e) {
          errback('CreateExtensionDirectoryError', e);
          return;
      }
      const [file, stream] = Gio2_0.File.new_tmp('XXXXXX.shell-extension.zip');
      const contents = message.response_body.flatten().get_as_bytes();
      stream.output_stream.write_bytes(contents, null);
      stream.close(null);
      const [success, pid] = GLib2_0.spawn_async(null, ['unzip', '-uod', dir.get_path(), '--', file.get_path()], null, GLib2_0.SpawnFlags.SEARCH_PATH | GLib2_0.SpawnFlags.DO_NOT_REAP_CHILD, null);
      if (!success) {
          errback('ExtractExtensionError');
          return;
      }
      if (pid) {
          GLib2_0.child_watch_add(GLib2_0.PRIORITY_DEFAULT, pid, (o, status) => {
              GLib2_0.spawn_close_pid(pid);
              if (status != 0)
                  errback('ExtractExtensionError');
              else
                  callback();
          });
      }
  };
  const installExtension = async (extensionId) => {
      return new Promise((resolve) => {
          const params = { shell_version: imports.misc.config.PACKAGE_VERSION };
          const url = `https://extensions.gnome.org/download-extension/${extensionId}.shell-extension.zip`;
          const message = Soup2_4.form_request_new_from_hash('GET', url, params);
          const dir = Gio2_0.File.new_for_path(GLib2_0.build_filenamev([GLib2_0.build_filenamev([GLib2_0.get_user_data_dir(), 'gnome-shell']), 'extensions', extensionId]));
          const callback = () => {
              try {
                  const extension = getExtensionManager().createExtensionObject(extensionId, dir, imports.misc.extensionUtils.ExtensionType.PER_USER);
                  debug$2(`installed new extension ${extensionId}`);
                  getExtensionManager().loadExtension(extension);
                  if (!getExtensionManager().enableExtension(extensionId))
                      throw new Error(`Cannot add ${extensionId} to enabled extensions gsettings key`);
              }
              catch (e) {
                  removeExtension(extensionId);
                  resolve();
                  return;
              }
              resolve();
          };
          const _httpSession = new Soup2_4.SessionAsync({ ssl_use_system_ca_file: true });
          Soup2_4.Session.prototype.add_feature.call(_httpSession, new Soup2_4.ProxyResolverDefault());
          _httpSession.queue_message(message, (session) => {
              gotExtensionZipFile(session, message, extensionId, dir, callback, resolve);
          });
      });
  };

  class ExtensionsDataProvider {
      async getData() {
          return getAllExtensionConfigData();
      }
      async useData(extensionData) {
          const downloadedExtensions = Object.keys(extensionData);
          const localExtensions = getExtensionIds();
          localExtensions.forEach((extensionId) => downloadedExtensions.indexOf(extensionId) < 0 && removeExtension(extensionId));
          await Promise.all(downloadedExtensions.map((extensionId) => {
              return Object.keys(extensionData[extensionId]).map((schemaPath) => {
                  return writeDconfData(schemaPath, extensionData[extensionId][schemaPath]);
              });
          }));
          await Promise.all(downloadedExtensions.map(async (extensionId) => localExtensions.indexOf(extensionId) < 0 && installExtension(extensionId)));
      }
      getName() {
          return 'extensions';
      }
  }

  const debug$3 = logger('keybindings-data-provider');
  const keyBindingsSchemaList = [
      '/org/gnome/mutter/keybindings/',
      '/org/gnome/mutter/wayland/keybindings/',
      '/org/gnome/desktop/wm/keybindings/',
      '/org/gnome/settings-daemon/plugins/media-keys/',
  ];
  class KeyBindingsDataProvider {
      async getData() {
          return keyBindingsSchemaList.reduce(async (acc, schema) => {
              try {
                  return Object.assign(Object.assign({}, (await acc)), { [schema]: await readDconfData(schema) });
              }
              catch (ex) {
                  debug$3(`cannot dump settings for ${schema}`);
              }
              return acc;
          }, Promise.resolve({}));
      }
      async useData(keyBindingsData) {
          await Promise.all(Object.keys(keyBindingsData).map((schemaPath) => {
              return writeDconfData(schemaPath, keyBindingsData[schemaPath]);
          }));
      }
      getName() {
          return 'keybindings';
      }
  }

  const debug$4 = logger('tweaks-data-provider');
  const tweaksSchemaList = [
      '/org/gnome/desktop/background/',
      '/org/gnome/desktop/calendar/',
      '/org/gnome/desktop/input-sources/',
      '/org/gnome/desktop/interface/',
      '/org/gnome/desktop/peripherals/',
      '/org/gnome/desktop/screensaver/',
      '/org/gnome/desktop/sound/',
      '/org/gnome/desktop/wm/preferences/',
      '/org/gnome/mutter/',
      '/org/gnome/settings-daemon/plugins/xsettings/',
  ];
  class TweaksDataProvider {
      async getData() {
          return tweaksSchemaList.reduce(async (acc, schema) => {
              try {
                  return Object.assign(Object.assign({}, (await acc)), { [schema]: await readDconfData(schema) });
              }
              catch (ex) {
                  debug$4(`cannot dump settings for ${schema}`);
              }
              return acc;
          }, Promise.resolve({}));
      }
      async useData(tweaksData) {
          await Promise.all(Object.keys(tweaksData).map((schemaPath) => {
              return writeDconfData(schemaPath, tweaksData[schemaPath]);
          }));
      }
      getName() {
          return 'tweaks';
      }
  }

  const debug$5 = logger('data');
  var DataOperationStatus;
  (function (DataOperationStatus) {
      DataOperationStatus[DataOperationStatus["SUCCESS"] = 0] = "SUCCESS";
      DataOperationStatus[DataOperationStatus["FAIL"] = 1] = "FAIL";
  })(DataOperationStatus || (DataOperationStatus = {}));
  var DataProviderType;
  (function (DataProviderType) {
      DataProviderType[DataProviderType["EXTENSIONS"] = 0] = "EXTENSIONS";
      DataProviderType[DataProviderType["KEYBINDINGS"] = 1] = "KEYBINDINGS";
      DataProviderType[DataProviderType["TWEAKS"] = 2] = "TWEAKS";
  })(DataProviderType || (DataProviderType = {}));
  class Data {
      constructor() {
          this.settings = getCurrentExtensionSettings();
          this.providers = this.createProviders();
          this.settings.connect('changed', this.updateProviders.bind(this));
      }
      async getSyncData() {
          const resultList = await Promise.all(this.providers.map(async (provider) => {
              return {
                  [provider.getName()]: await provider.getData(),
              };
          }));
          return resultList.reduce((acc, result) => {
              return Object.assign(Object.assign({}, acc), result);
          }, { extensions: {}, keybindings: {}, tweaks: {} });
      }
      async use(syncData) {
          await Promise.all(this.providers.map(async (provider) => {
              debug$5(`updating ${provider.getName()} settings in local machine`);
              await provider.useData(syncData[provider.getName()]);
          }));
      }
      createProvider(providerType) {
          switch (providerType) {
              case DataProviderType.EXTENSIONS: {
                  return new ExtensionsDataProvider();
              }
              case DataProviderType.KEYBINDINGS: {
                  return new KeyBindingsDataProvider();
              }
              case DataProviderType.TWEAKS: {
                  return new TweaksDataProvider();
              }
          }
      }
      createProviders() {
          const providerFlag = this.settings.get_flags('data-providers');
          const providerTypes = settingsFlagsToEnumList(providerFlag);
          debug$5(`enabled data providers are ${providerTypes.map((p) => DataProviderType[p])}`);
          return providerTypes
              .map((providerType) => this.createProvider(providerType))
              .filter((provider) => provider !== undefined);
      }
      updateProviders() {
          this.providers = this.createProviders();
      }
  }

  const { Button } = imports.ui.panelMenu;
  const { PopupImageMenuItem, PopupSeparatorMenuItem } = imports.ui.popupMenu;
  const { panel } = imports.ui.main;
  const debug$6 = logger('statusMenu');
  class StatusMenu {
      constructor(eventEmitter) {
          this.eventEmitter = eventEmitter;
          this.extension = getCurrentExtension();
          this.settings = getCurrentExtensionSettings();
      }
      show() {
          if (this.button === undefined) {
              this.button = this.createButton();
          }
          this.eventEmitter.on(ApiEvent.UPLOAD, this.disableButton.bind(this));
          this.eventEmitter.on(ApiEvent.DOWNLOAD, this.disableButton.bind(this));
          this.eventEmitter.on(ApiEvent.UPLOAD_FINISHED, this.enableButton.bind(this));
          this.eventEmitter.on(ApiEvent.DOWNLOAD_FINISHED, this.enableButton.bind(this));
          panel.addToStatusArea('extensions-sync', this.button);
          debug$6('showing status menu in panel');
      }
      hide() {
          if (this.button) {
              this.button.destroy();
              this.button = undefined;
          }
          if (panel.statusArea['extensions-sync']) {
              panel.statusArea['extensions-sync'].destroy();
              this.eventEmitter.off(ApiEvent.UPLOAD, this.disableButton.bind(this));
              this.eventEmitter.off(ApiEvent.DOWNLOAD, this.disableButton.bind(this));
              this.eventEmitter.off(ApiEvent.UPLOAD_FINISHED, this.enableButton.bind(this));
              this.eventEmitter.off(ApiEvent.DOWNLOAD_FINISHED, this.enableButton.bind(this));
              debug$6('removing status menu from panel');
          }
      }
      createButton() {
          const newButton = new Button(0, _('Sync Settings'));
          newButton.icon = this.createIcon('synced');
          newButton.add_actor(newButton.icon);
          newButton.menu.addMenuItem(this.createMenuItem(_('Upload'), 'upload', () => this.eventEmitter.emit(ApiEvent.UPLOAD)));
          newButton.menu.addMenuItem(this.createMenuItem(_('Download'), 'download', () => this.eventEmitter.emit(ApiEvent.DOWNLOAD)));
          newButton.menu.addMenuItem(new PopupSeparatorMenuItem());
          newButton.menu.addMenuItem(this.createMenuItem(_('Preferences'), 'preferences', () => {
              execute(`gnome-extensions prefs "${this.extension.metadata.uuid}"`);
          }));
          return newButton;
      }
      createMenuItem(menuTitle, actionType, onClick) {
          const menuItem = new PopupImageMenuItem(`${menuTitle}`, this.createIcon(`${actionType}`).gicon);
          if (onClick) {
              menuItem.connect('activate', () => onClick());
          }
          return menuItem;
      }
      createIcon(iconType) {
          return new St1_0.Icon({
              gicon: Gio2_0.icon_new_for_string(`${this.extension.path}/icons/extensions-sync-${iconType}-symbolic.svg`),
              style_class: 'system-status-icon',
          });
      }
      enableButton() {
          if (this.button !== undefined) {
              this.button.set_reactive(true);
              this.button.icon.set_gicon(this.createIcon('synced').gicon);
          }
      }
      disableButton() {
          if (this.button !== undefined) {
              this.button.set_reactive(false);
              this.button.icon.set_gicon(this.createIcon('syncing').gicon);
              this.button.menu.close();
          }
      }
  }

  var SyncEvents;
  (function (SyncEvents) {
      SyncEvents[SyncEvents["SYNCHRONIZED"] = 0] = "SYNCHRONIZED";
  })(SyncEvents || (SyncEvents = {}));
  const debug$7 = logger('sync');
  class Sync {
      constructor(eventEmitter, data) {
          this.data = data;
          this.eventEmitter = eventEmitter;
      }
      start() {
          this.eventEmitter.on(ApiEvent.DOWNLOAD_FINISHED, this.onDownloadFinished.bind(this));
          debug$7('listening for download completion events');
      }
      stop() {
          this.eventEmitter.off(ApiEvent.DOWNLOAD_FINISHED, this.onDownloadFinished.bind(this));
          debug$7('stopped listening for download completion events');
      }
      async onDownloadFinished(syncData) {
          if (syncData === undefined) {
              return;
          }
          try {
              await this.data.use(syncData);
          }
          catch (ex) {
              notify(_('Failed to apply sync data to current system.'));
              debug$7(`failed to apply sync data to system: ${ex}`);
          }
          if (canRestartShell()) {
              restartShell(_('Extensions are updated. Reloading Gnome Shell'));
          }
          else {
              notify(_('Extensions are updated. Please reload Gnome Shell'));
          }
      }
  }

  // Copyright Joyent, Inc. and other Node contributors.

  var R = typeof Reflect === 'object' ? Reflect : null;
  var ReflectApply = R && typeof R.apply === 'function'
    ? R.apply
    : function ReflectApply(target, receiver, args) {
      return Function.prototype.apply.call(target, receiver, args);
    };

  var ReflectOwnKeys;
  if (R && typeof R.ownKeys === 'function') {
    ReflectOwnKeys = R.ownKeys;
  } else if (Object.getOwnPropertySymbols) {
    ReflectOwnKeys = function ReflectOwnKeys(target) {
      return Object.getOwnPropertyNames(target)
        .concat(Object.getOwnPropertySymbols(target));
    };
  } else {
    ReflectOwnKeys = function ReflectOwnKeys(target) {
      return Object.getOwnPropertyNames(target);
    };
  }

  function ProcessEmitWarning(warning) {
    if (console && console.warn) console.warn(warning);
  }

  var NumberIsNaN = Number.isNaN || function NumberIsNaN(value) {
    return value !== value;
  };

  function EventEmitter() {
    EventEmitter.init.call(this);
  }
  var events = EventEmitter;
  var once_1 = once;

  // Backwards-compat with node 0.10.x
  EventEmitter.EventEmitter = EventEmitter;

  EventEmitter.prototype._events = undefined;
  EventEmitter.prototype._eventsCount = 0;
  EventEmitter.prototype._maxListeners = undefined;

  // By default EventEmitters will print a warning if more than 10 listeners are
  // added to it. This is a useful default which helps finding memory leaks.
  var defaultMaxListeners = 10;

  function checkListener(listener) {
    if (typeof listener !== 'function') {
      throw new TypeError('The "listener" argument must be of type Function. Received type ' + typeof listener);
    }
  }

  Object.defineProperty(EventEmitter, 'defaultMaxListeners', {
    enumerable: true,
    get: function() {
      return defaultMaxListeners;
    },
    set: function(arg) {
      if (typeof arg !== 'number' || arg < 0 || NumberIsNaN(arg)) {
        throw new RangeError('The value of "defaultMaxListeners" is out of range. It must be a non-negative number. Received ' + arg + '.');
      }
      defaultMaxListeners = arg;
    }
  });

  EventEmitter.init = function() {

    if (this._events === undefined ||
        this._events === Object.getPrototypeOf(this)._events) {
      this._events = Object.create(null);
      this._eventsCount = 0;
    }

    this._maxListeners = this._maxListeners || undefined;
  };

  // Obviously not all Emitters should be limited to 10. This function allows
  // that to be increased. Set to zero for unlimited.
  EventEmitter.prototype.setMaxListeners = function setMaxListeners(n) {
    if (typeof n !== 'number' || n < 0 || NumberIsNaN(n)) {
      throw new RangeError('The value of "n" is out of range. It must be a non-negative number. Received ' + n + '.');
    }
    this._maxListeners = n;
    return this;
  };

  function _getMaxListeners(that) {
    if (that._maxListeners === undefined)
      return EventEmitter.defaultMaxListeners;
    return that._maxListeners;
  }

  EventEmitter.prototype.getMaxListeners = function getMaxListeners() {
    return _getMaxListeners(this);
  };

  EventEmitter.prototype.emit = function emit(type) {
    var args = [];
    for (var i = 1; i < arguments.length; i++) args.push(arguments[i]);
    var doError = (type === 'error');

    var events = this._events;
    if (events !== undefined)
      doError = (doError && events.error === undefined);
    else if (!doError)
      return false;

    // If there is no 'error' event listener then throw.
    if (doError) {
      var er;
      if (args.length > 0)
        er = args[0];
      if (er instanceof Error) {
        // Note: The comments on the `throw` lines are intentional, they show
        // up in Node's output if this results in an unhandled exception.
        throw er; // Unhandled 'error' event
      }
      // At least give some kind of context to the user
      var err = new Error('Unhandled error.' + (er ? ' (' + er.message + ')' : ''));
      err.context = er;
      throw err; // Unhandled 'error' event
    }

    var handler = events[type];

    if (handler === undefined)
      return false;

    if (typeof handler === 'function') {
      ReflectApply(handler, this, args);
    } else {
      var len = handler.length;
      var listeners = arrayClone(handler, len);
      for (var i = 0; i < len; ++i)
        ReflectApply(listeners[i], this, args);
    }

    return true;
  };

  function _addListener(target, type, listener, prepend) {
    var m;
    var events;
    var existing;

    checkListener(listener);

    events = target._events;
    if (events === undefined) {
      events = target._events = Object.create(null);
      target._eventsCount = 0;
    } else {
      // To avoid recursion in the case that type === "newListener"! Before
      // adding it to the listeners, first emit "newListener".
      if (events.newListener !== undefined) {
        target.emit('newListener', type,
                    listener.listener ? listener.listener : listener);

        // Re-assign `events` because a newListener handler could have caused the
        // this._events to be assigned to a new object
        events = target._events;
      }
      existing = events[type];
    }

    if (existing === undefined) {
      // Optimize the case of one listener. Don't need the extra array object.
      existing = events[type] = listener;
      ++target._eventsCount;
    } else {
      if (typeof existing === 'function') {
        // Adding the second element, need to change to array.
        existing = events[type] =
          prepend ? [listener, existing] : [existing, listener];
        // If we've already got an array, just append.
      } else if (prepend) {
        existing.unshift(listener);
      } else {
        existing.push(listener);
      }

      // Check for listener leak
      m = _getMaxListeners(target);
      if (m > 0 && existing.length > m && !existing.warned) {
        existing.warned = true;
        // No error code for this since it is a Warning
        // eslint-disable-next-line no-restricted-syntax
        var w = new Error('Possible EventEmitter memory leak detected. ' +
                            existing.length + ' ' + String(type) + ' listeners ' +
                            'added. Use emitter.setMaxListeners() to ' +
                            'increase limit');
        w.name = 'MaxListenersExceededWarning';
        w.emitter = target;
        w.type = type;
        w.count = existing.length;
        ProcessEmitWarning(w);
      }
    }

    return target;
  }

  EventEmitter.prototype.addListener = function addListener(type, listener) {
    return _addListener(this, type, listener, false);
  };

  EventEmitter.prototype.on = EventEmitter.prototype.addListener;

  EventEmitter.prototype.prependListener =
      function prependListener(type, listener) {
        return _addListener(this, type, listener, true);
      };

  function onceWrapper() {
    if (!this.fired) {
      this.target.removeListener(this.type, this.wrapFn);
      this.fired = true;
      if (arguments.length === 0)
        return this.listener.call(this.target);
      return this.listener.apply(this.target, arguments);
    }
  }

  function _onceWrap(target, type, listener) {
    var state = { fired: false, wrapFn: undefined, target: target, type: type, listener: listener };
    var wrapped = onceWrapper.bind(state);
    wrapped.listener = listener;
    state.wrapFn = wrapped;
    return wrapped;
  }

  EventEmitter.prototype.once = function once(type, listener) {
    checkListener(listener);
    this.on(type, _onceWrap(this, type, listener));
    return this;
  };

  EventEmitter.prototype.prependOnceListener =
      function prependOnceListener(type, listener) {
        checkListener(listener);
        this.prependListener(type, _onceWrap(this, type, listener));
        return this;
      };

  // Emits a 'removeListener' event if and only if the listener was removed.
  EventEmitter.prototype.removeListener =
      function removeListener(type, listener) {
        var list, events, position, i, originalListener;

        checkListener(listener);

        events = this._events;
        if (events === undefined)
          return this;

        list = events[type];
        if (list === undefined)
          return this;

        if (list === listener || list.listener === listener) {
          if (--this._eventsCount === 0)
            this._events = Object.create(null);
          else {
            delete events[type];
            if (events.removeListener)
              this.emit('removeListener', type, list.listener || listener);
          }
        } else if (typeof list !== 'function') {
          position = -1;

          for (i = list.length - 1; i >= 0; i--) {
            if (list[i] === listener || list[i].listener === listener) {
              originalListener = list[i].listener;
              position = i;
              break;
            }
          }

          if (position < 0)
            return this;

          if (position === 0)
            list.shift();
          else {
            spliceOne(list, position);
          }

          if (list.length === 1)
            events[type] = list[0];

          if (events.removeListener !== undefined)
            this.emit('removeListener', type, originalListener || listener);
        }

        return this;
      };

  EventEmitter.prototype.off = EventEmitter.prototype.removeListener;

  EventEmitter.prototype.removeAllListeners =
      function removeAllListeners(type) {
        var listeners, events, i;

        events = this._events;
        if (events === undefined)
          return this;

        // not listening for removeListener, no need to emit
        if (events.removeListener === undefined) {
          if (arguments.length === 0) {
            this._events = Object.create(null);
            this._eventsCount = 0;
          } else if (events[type] !== undefined) {
            if (--this._eventsCount === 0)
              this._events = Object.create(null);
            else
              delete events[type];
          }
          return this;
        }

        // emit removeListener for all listeners on all events
        if (arguments.length === 0) {
          var keys = Object.keys(events);
          var key;
          for (i = 0; i < keys.length; ++i) {
            key = keys[i];
            if (key === 'removeListener') continue;
            this.removeAllListeners(key);
          }
          this.removeAllListeners('removeListener');
          this._events = Object.create(null);
          this._eventsCount = 0;
          return this;
        }

        listeners = events[type];

        if (typeof listeners === 'function') {
          this.removeListener(type, listeners);
        } else if (listeners !== undefined) {
          // LIFO order
          for (i = listeners.length - 1; i >= 0; i--) {
            this.removeListener(type, listeners[i]);
          }
        }

        return this;
      };

  function _listeners(target, type, unwrap) {
    var events = target._events;

    if (events === undefined)
      return [];

    var evlistener = events[type];
    if (evlistener === undefined)
      return [];

    if (typeof evlistener === 'function')
      return unwrap ? [evlistener.listener || evlistener] : [evlistener];

    return unwrap ?
      unwrapListeners(evlistener) : arrayClone(evlistener, evlistener.length);
  }

  EventEmitter.prototype.listeners = function listeners(type) {
    return _listeners(this, type, true);
  };

  EventEmitter.prototype.rawListeners = function rawListeners(type) {
    return _listeners(this, type, false);
  };

  EventEmitter.listenerCount = function(emitter, type) {
    if (typeof emitter.listenerCount === 'function') {
      return emitter.listenerCount(type);
    } else {
      return listenerCount.call(emitter, type);
    }
  };

  EventEmitter.prototype.listenerCount = listenerCount;
  function listenerCount(type) {
    var events = this._events;

    if (events !== undefined) {
      var evlistener = events[type];

      if (typeof evlistener === 'function') {
        return 1;
      } else if (evlistener !== undefined) {
        return evlistener.length;
      }
    }

    return 0;
  }

  EventEmitter.prototype.eventNames = function eventNames() {
    return this._eventsCount > 0 ? ReflectOwnKeys(this._events) : [];
  };

  function arrayClone(arr, n) {
    var copy = new Array(n);
    for (var i = 0; i < n; ++i)
      copy[i] = arr[i];
    return copy;
  }

  function spliceOne(list, index) {
    for (; index + 1 < list.length; index++)
      list[index] = list[index + 1];
    list.pop();
  }

  function unwrapListeners(arr) {
    var ret = new Array(arr.length);
    for (var i = 0; i < ret.length; ++i) {
      ret[i] = arr[i].listener || arr[i];
    }
    return ret;
  }

  function once(emitter, name) {
    return new Promise(function (resolve, reject) {
      function eventListener() {
        if (errorListener !== undefined) {
          emitter.removeListener('error', errorListener);
        }
        resolve([].slice.call(arguments));
      }    var errorListener;

      // Adding an error listener is not optional because
      // if an error is thrown on an event emitter we cannot
      // guarantee that the actual event we are waiting will
      // be fired. The result could be a silent way to create
      // memory or file descriptor leaks, which is something
      // we should avoid.
      if (name !== 'error') {
        errorListener = function errorListener(err) {
          emitter.removeListener(name, eventListener);
          reject(err);
        };

        emitter.once('error', errorListener);
      }

      emitter.once(name, eventListener);
    });
  }
  events.once = once_1;

  const debug$8 = logger('extension');
  class SyncExtension {
      constructor() {
          this.eventEmitter = new events.EventEmitter();
          this.data = new Data();
          this.api = new Api(this.eventEmitter, this.data);
          this.sync = new Sync(this.eventEmitter, this.data);
          this.statusMenu = new StatusMenu(this.eventEmitter);
          debug$8('extension is initialized');
      }
      enable() {
          this.sync.start();
          this.statusMenu.show();
          debug$8('extension is enabled');
      }
      disable() {
          this.sync.stop();
          this.statusMenu.hide();
          debug$8('extension is disabled');
      }
  }
  function extension () {
      return new SyncExtension();
  }

  return extension;

}(imports.gi.Gio, imports.gi.GLib, imports.gi.Soup, imports.gi.St));
